;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Magikoopa's Magic (sprite 20), by imamelia
;;
;; This is a disassembly of sprite 20 in SMW, the magic that Magikoopa shoots.
;;
;; Uses first extra bit: NO
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; defines and tables
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

MagicPalette:
db $05,$07,$09,$0B

Displacement:
db $00,$01,$02,$05,$08,$0B,$0E,$0F
db $10,$0F,$0E,$0B,$08,$05,$02,$01

!Tile1 = $88
!Tile2 = $89
!Tile3 = $98

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; init routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "INIT ",pc
RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; main routine wrapper
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "MAIN ",pc
PHB
PHK
PLB
JSR MagikoopaMagicMain
PLB
RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; main routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

MagikoopaMagicMain:

LDA $9D		; if sprites are not locked...
BEQ NormalRt	; run the normal code
JMP AlternateMain	; else, run the alternate code

NormalRt:		;

JSR ShowStars	; show some small star graphics

JSL $01801A	; update sprite Y position without gravity
JSL $018022	; update sprite X position without gravity

LDA $AA,x	;
PHA		; preserve the sprite's Y speed
LDA #$FF		;
STA $AA,x	; so we can temporarily set it to FF
JSL $019138	; for the object contact routine
PLA		;
STA $AA,x	;

LDA $1588,x	;
AND #$08	; if the sprite is not touching the ceiling...
BEQ AlternateMain	;
LDA $15A0,x	; or it is offscreen horizontally...
BNE AlternateMain	; skip to the other part of the main routine

LDA #$01		;
STA $1DF9	; play the "hit block" sound effect
STZ $14C8,x	; erase the sprite

LDA $185F	; check the tile number that the sprite is touching
SEC		;
SBC #$11		;
CMP #$1D	; if it is not between a certain range (111-12D)...
BCS NoGen	; don't erase it or generate a sprite
JSL $01ACF9	; get a random number
ADC $148E	;
ADC $7B		; add a little more randomness to the mix
ADC $13		;

LDY #$78		; 78 = 1-Up mushroom
CMP #$35		; if our random number was 35...
BEQ StoreSpriteNum	; then generate a 1-Up mushroom
LDY #$21		; 21 = moving coin
CMP #$08		; if our random number was less than 08...
BCC StoreSpriteNum	; then generate a moving coin
LDY #$27		; 27 = Thwimp
CMP #$F7		; if our random number was F7 or greater...
BCS StoreSpriteNum	; then generate a Thwimp
LDY #$07		; 07 = yellow Koopa
StoreSpriteNum:	; if our random number was anything else...
STY $9E,x		; then generate a yellow Koopa

LDA #$08		; normal status
STA $14C8,x	;

JSL $07F7D2	; initialize sprite tables

LDA $9B		; block X position high byte
STA $14E0,x	; sprite X position high byte
LDA $9A		; block X position low byte
AND #$F0	; directly on a tile
STA $E4,x		; sprite X position low byte
LDA $99		; block Y position high byte
STA $14D4,x	; sprite Y position high byte
LDA $98		; block Y position low byte
AND #$F0	; directly on a tile
STA $D8,x	; sprite Y position low byte

LDA #$02		; tile to generate = blank tile
STA $9C		;

JSL $00BEB0	; generate a tile

NoGen:		;

JSR MakeSmoke	;

RTS		;

AlternateMain:

JSL $01803A	; interact with the player and with other sprites

LDA $13		;
LSR		;
LSR		;
AND #$03	; palette index
TAY		;
LDA MagicPalette,y	; change the palette of the magic every 4 frames
STA $15F6,x	;

JSR MagikoopaMagicGFX

LDA $D8,x	;
SEC		;
SBC $1C		;
CMP #$E0		; if the sprite is too far offscreen...
BCC NoStars	;
STZ $14C8,x	; erase it

NoStars:		;
RTS

ShowStars:	;

LDA $13		;
AND #$03	; if the frame isn't a multiple of 4...
ORA $186C,x	; the sprite is offscreen vertically...
ORA $9D		; or sprites are locked...
BNE NoStars	; don't show any stars

JSL $01ACF9	; get a random number
AND #$0F	; between 00 and 0F

CLC		;
LDY #$00		;
ADC #$FC	;
BPL NoDecY1	;
DEY		;
NoDecY1:		;
CLC		;
ADC $E4,x	; set the position of the stars
STA $02		;
TYA		;
ADC $14E0,x	;
PHA		;
LDA $02		;
CMP $1A		;
PLA		;
SBC $1B		;
BNE NoStars	;

LDA $148E	;
AND #$0F	;
CLC		;
ADC #$FE		;
ADC $D8,x	;
STA $00		;
LDA $14D4,x	;
ADC #$00	;
STA $01		;

JSL $0285BA	; show small stars routine

RTS		;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; graphics routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

MagikoopaMagicGFX:

JSR GetDrawInfo

LDA $14		; some preliminary stuff
LSR		;
AND #$0F	; get an index for the X displacement
STA $03		;
CLC		;
ADC #$0C	; get an index for the Y displacement
AND #$0F	;
STA $02		;

LDA $01		;
SEC		;
SBC #$04		; offset the sprite's X and Y position
STA $01		;
LDA $00		; by -4 each
SEC		;
SBC #$04		;
STA $00		;

LDX $02			;
LDA $01			;
CLC			;
ADC Displacement,x	; set the Y displacement of the first tile
STA $0301,y		;

LDX $03
LDA $00			;
CLC			;
ADC Displacement,x	; set the X displacement of the first tile
STA $0300,y		;

LDA $02			;
CLC			;
ADC #$05		; Y index 5 places down
AND #$0F		;
STA $02			;

TAX			;
LDA $01			;
CLC			;
ADC Displacement,x	; set the Y displacement of the second tile
STA $0305,y		;

LDA $03			;
CLC			;
ADC #$05		; X index 5 places down
AND #$0F		;
STA $03			;

LDX $03
LDA $00			;
CLC			;
ADC Displacement,x	; set the X displacement of the second tile
STA $0304,y		;

LDA $02			;
CLC			;
ADC #$05		; Y index 5 places down
AND #$0F		;
STA $02			;

TAX			;
LDA $01			;
CLC			;
ADC Displacement,x	; set the Y displacement of the third tile
STA $0309,y		;

LDA $03			;
CLC			;
ADC #$05		; X index 5 places down
AND #$0F		;
STA $03			;

LDX $03
LDA $00			;
CLC			;
ADC Displacement,x	; set the X displacement of the third tile
STA $0308,y		;

LDX $15E9		;
LDA $15F6,x		; sprite palette/GFX page
ORA $64			; add in sprite priority
STA $0303,y		; tile properties for the first tile
STA $0307,y		; tile properties for the second tile
STA $030B,y		; tile properties for the third tile

LDA #!Tile1		;
STA $0302,y		; first tile number
LDA #!Tile2		;
STA $0306,y		; second tile number
LDA #!Tile3		;
STA $030A,y		; third tile number

LDY #$00			; 8x8 tiles
LDA #$02			; 3 of them
JSL $81B7B3		;

RTS			;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; miscellaneous subroutines
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Table1:              db $0C,$1C
Table2:              db $01,$02
Table3:              db $40,$B0
Table6:              db $01,$FF
Table4:              db $30,$C0,$A0,$C0,$A0,$F0,$60,$90,$30,$C0,$A0,$80,$A0,$40,$60,$B0
Table5:              db $01,$FF,$01,$FF,$01,$FF,$01,$FF,$01,$FF,$01,$FF,$01,$00,$01,$FF

SubOffscreenX0:
LDA #$00
;BRA SubOffscreenMain
;SubOffscreenX1:
;LDA #$02
;BRA SubOffscreenMain
;SubOffscreenX2:
;LDA #$04
;BRA SubOffscreenMain
;SubOffscreenX3:
;LDA #$06
;BRA SubOffscreenMain
;SubOffscreenX4:
;LDA #$08
;BRA SubOffscreenMain
;SubOffscreenX5:
;LDA #$0A
;BRA SubOffscreenMain
;SubOffscreenX6:
;LDA #$0C
;BRA SubOffscreenMain
;SubOffscreenX7:
;LDA #$0E

SubOffscreenMain:

STA $03

JSR SubIsOffscreen
BEQ Return2

LDA $5B
LSR
BCS VerticalLevel
LDA $D8,x
CLC
ADC #$50
LDA $14D4,x
ADC #$00
CMP #$02
BPL EraseSprite
LDA $167A,x
AND #$04
BNE Return2
LDA $13
AND #$01
ORA $03
STA $01
TAY
LDA $1A
CLC
ADC Table4,y
ROL $00
CMP $E4,x
PHP
LDA $1B
LSR $00
ADC Table5,y
PLP
SBC $14E0,x
STA $00
LSR $01
BCC Label20
EOR #$80
STA $00
Label20:
LDA $00
BPL Return2

EraseSprite:
LDA $14C8,x
CMP #$08
BCC KillSprite
LDY $161A,x
CPY #$FF
BEQ KillSprite
LDA #$00
STA $1938,y
KillSprite:
STZ $14C8,x
Return2:
RTS

VerticalLevel:

LDA $167A,x
AND #$04
BNE Return2
LDA $13
LSR
BCS Return2
AND #$01
STA $01
TAY
LDA $1C
CLC
ADC Table3,y
ROL $00
CMP $D8,x
PHP
LDA $1D
LSR $00
ADC Table6,y
PLP
SBC $14D4,x
STA $00
LDY $02
BEQ Label22
EOR #$80
STA $00
Label22:
LDA $00
BPL Return2
BMI EraseSprite

SubIsOffscreen:
LDA $15A0,x
ORA $186C,x
RTS

GetDrawInfo:

STZ $186C,x
STZ $15A0,x
LDA $E4,x
CMP $1A
LDA $14E0,x
SBC $1B
BEQ OnscreenX
INC $15A0,x
OnscreenX:
LDA $14E0,x
XBA
LDA $E4,x
REP #$20
SEC
SBC $1A
CLC
ADC.w #$0040
CMP #$0180
SEP #$20
ROL A
AND #$01
STA $15C4,x
BNE Invalid

LDY #$00
LDA $1662,x
AND #$20
BEQ OnscreenLoop
INY
OnscreenLoop:
LDA $D8,x
CLC
ADC Table1,y
PHP
CMP $1C
ROL $00
PLP
LDA $14D4,x
ADC #$00
LSR $00
SBC $1D
BEQ OnscreenY
LDA $186C,x
ORA Table2,y
STA $186C,x
OnscreenY:
DEY
BPL OnscreenLoop
LDY $15EA,x
LDA $E4,x
SEC
SBC $1A
STA $00
LDA $D8,x
SEC
SBC $1C
STA $01
RTS

Invalid:
PLA
PLA
RTS

MakeSmoke:

LDY #$03		; find a free slot to display smoke effect
SmokeSlotLoop:	;
LDA $17C0,y	; if this slot is free...
BEQ GenSmoke	; generate some smoke
DEY		;
BPL SmokeSlotLoop	; check all 4 slots
RTS

GenSmoke:	;

LDA #$01		; 01 = puff of smoke
STA $17C0,y	;
LDA $E4,x	; sprite X position low byte
STA $17C8,y	; smoke effect X position
LDA $D8,x	; sprite Y position low byte
STA $17C4,y	; smoke effect Y position
LDA #$1B		; time to show smoke
STA $17CC,y	;

RTS


